// native int uint(int a, int op, int b);
static cell_t native_uint(IPluginContext *pContext, const cell_t *params)
{
	// -5 = 4294967291, so it's a bad idea: uint(10, '+', -5).
	// Don't pass a negative number if you don't know what you are doing.
	// If the number has exceeded int limits (int max 2147483647), then here we get it negative, because 2147483650 in SourcePawn == -2147483646.
	uint32_t a = static_cast<uint32_t>(params[1]);
	uint32_t b = static_cast<uint32_t>(params[3]);
	switch (static_cast<char>(params[2])) {
		case '+': {
			uint32_t i = a + b;
			return i >= a ? i : UINT_MAX_; // UINT_MAX_ + 1 = UINT_MAX_
		}
		case '-': {
			uint32_t i = a - b;
			return i <= a ? i : 0 ; // 0 - 1 = 0
		}
		case '*': {
			if (!a || !b) {
				return 0;
			}
			return b > (UINT_MAX_ / a) ? UINT_MAX_ : (a * b);
		}
		case '/': {
			return b ? (a / b) : 0;
		}
		case '%': {
			return b ? (a % b) : 0;
		}
	}
	return 0;
}

// bool b = uintcmp(5, 3) >= 0; // true, if 5 >= 3 (instead of operator >=, use any. 0 must be always).
static cell_t native_uintcmp(IPluginContext *pContext, const cell_t *params)
{
	uint32_t a = static_cast<uint32_t>(params[1]);
	uint32_t b = static_cast<uint32_t>(params[2]);
	if (a < b) {
		return -1;
	}
	if (a == b) {
		return 0;
	}
	return 1; // a > b
}

// -----

static bool LeapYear(int y) {
	if (y % 4) {
		return false;
	}
	if (y % 100) {
		return true;
	}
	return !(y % 400);
}

static int days_in_a_month[12] = 
{
	31, /* January */
	28, /* February (29 if leap year) */
	31, /* March */
	30, /* April */
	31, /* May */
	30, /* June */
	31, /* July */
	31, /* August */
	30, /* September */
	31, /* October */
	30, /* November */
	31  /* December */
};

static void days_in_a_month_update(int year) {
	days_in_a_month[1] = LeapYear(year) ? 29 : 28;
}

/*
tm_wday
0	Sunday		Sun	Su	Воскресенье
1	Monday		Mon	Mo	Понедельник
2	Tuesday		Tue	Tu	Вторник
3	Wednesday	Wed	We	Среда
4	Thursday	Thu	Th	Четверг
5	Friday		Fri	Fr	Пятница
6	Saturday	Sat	Sa	Суббота
*/
static void add_day(struct tm &t) {
	if (++t.tm_wday == 7 ) {
		t.tm_wday = 0;
	}
	if (++t.tm_mday > days_in_a_month[t.tm_mon]) {
		t.tm_mday = 1;
		if (++t.tm_mon == 12)
		{
			t.tm_mon  = 0;
			t.tm_yday = 0;
			days_in_a_month_update(++t.tm_year);
			return;
		}
	}
	++t.tm_yday;
}

// native bool FormatUTime(char[] buffer, int maxlength, const char[] format, int stamp=0);
static cell_t native_FormatUTime(IPluginContext *pContext, const cell_t *params)
{
	char *buffer, *format;
	pContext->LocalToString(params[1], &buffer);
	pContext->LocalToStringNULL(params[3], &format);
	
	// Seconds have passed since January 1, 1970.
	uint32_t x_timestamp = !params[4] ? g_pSM->GetAdjustedTime() : static_cast<uint32_t>(params[4]);
	
	#if UINT_DBG_
	uint32_t iterations = 0; // 4294967295 (max) x_timestamp = 49759 iterations
	g_pSM->LogMessage(myself, "\n \n-------------------------------\nx_timestamp: %u", x_timestamp);
	#endif
	
	// 0 timestamp = Thursday, 01 January 1970 00:00:00
	struct tm t;
	t.tm_sec 	= 0; // Seconds (0-60)
	t.tm_min 	= 0; // Minutes (0-59)
	t.tm_hour 	= 0; // Hours (0-23)
	t.tm_mday 	= 1; // Day of the month (1-31)
	t.tm_mon 	= 0; // Month (0-11)
	t.tm_year	= 1970; // Year - 1900
	t.tm_wday	= 4; // Day of the week (0-6, Sunday = 0)
	t.tm_yday	= 0; // Day in the year (0-365, 1 Jan = 0)
	t.tm_isdst	= 0; // Daylight saving time
	
	days_in_a_month_update(t.tm_year);
	
	// Reducing the number of iterations by adding days
	while (x_timestamp >= 86400)
	{
		#if UINT_DBG_
		++iterations;
		#endif
		
		x_timestamp -= 86400;
		add_day(t);
	}
	
	// Reducing the number of iterations by adding hours
	while (x_timestamp >= 3600)
	{
		#if UINT_DBG_
		++iterations;
		#endif
		
		x_timestamp -= 3600;
		if (++t.tm_hour == 24) {
			t.tm_hour = 0;
			add_day(t);
		}
	}
	
	// Reducing the number of iterations by adding minutes
	while (x_timestamp >= 60)
	{
		#if UINT_DBG_
		++iterations;
		#endif
		
		x_timestamp -= 60;
		if (++t.tm_min == 60) {
			t.tm_min = 0;
			if (++t.tm_hour == 24) {
				t.tm_hour = 0;
				add_day(t);
			}
		}
	}
	
	// Adding remaining seconds
	while (x_timestamp--)
	{
		#if UINT_DBG_
		++iterations;
		#endif
		
		if (++t.tm_sec == 60) {
			t.tm_sec = 0;
			if (++t.tm_min == 60) {
				t.tm_min = 0;
				if (++t.tm_hour == 24) {
					t.tm_hour = 0;
					add_day(t);
				}
			}
		}
	}
	
	//
	#if UINT_DBG_
	g_pSM->LogMessage(myself, "\niterations: %u\n \nt.tm_sec\t%d\nt.tm_min\t%d\nt.tm_hour\t%d\nt.tm_mday\t%d\nt.tm_mon\t%d\nt.tm_year\t%d\nt.tm_wday\t%d\nt.tm_yday\t%d\n",
		iterations, t.tm_sec, t.tm_min, t.tm_hour, t.tm_mday, t.tm_mon, t.tm_year, t.tm_wday, t.tm_yday);
	#endif
	//
	
	t.tm_year -= 1900;
	
	if (strftime(buffer, params[2], format, (const tm *)&t))
	{
		#if UINT_DBG_
		g_pSM->LogMessage(myself, "\n%s\n", buffer);
		#endif
		
		return 1;
	}
	
	#if UINT_DBG_
	g_pSM->LogMessage(myself, "\nerror");
	#endif
	
	buffer[0] = '\0';
	return 0;
}